function Data=create_data_struct(filename, do_not_load_test_data, read_as_sparse, add_valid)
%Data=create_data_struct(filename, do_not_load_test_data, read_as_spars, add_valid)
% Input:
% filename -- Trunk name of the Data files (including directory path), with
%               no extension.
% do_not_load_test_data -- 0/1 flag instructing not to load the test data
% read_as_sparse -- Read matrix as sparse matrix
% Returns:
% Data     -- A structure with 3 fields 'train', 'valid', and 'test'
%               containing Data objects in the spider format.

% Isabelle Guyon -- isabelle@clopinet.com -- October 2007

if nargin<2 | isempty(do_not_load_test_data), do_not_load_test_data=0; end
if nargin<3 | isempty(read_as_sparse), read_as_sparse=[]; end
if nargin<4, add_valid=0; end
    
p   = read_parameters(filename);
print_parameters(p);

% Read the column info (feature names) if available
Fid=[];
if exist([filename '_feat.id'], 'file')
    Fid=textread([filename '_feat.id'], '%s');
end
Fsource=[];
if exist([filename '_feat.source'], 'file')
    Fsource=textread([filename '_feat.source'], '%s');
end
Fcauses=[];
if exist([filename '_feat.causes'], 'file')
    Fcauses=load([filename '_feat.causes']);
end
Fdcauses=[];
if exist([filename '_feat.dcauses'], 'file')
    Fdcauses=load([filename '_feat.dcauses']);
end
Feffects=[];
if exist([filename '_feat.effects'], 'file')
    Feffects=load([filename '_feat.effects']);
end
Fdeffects=[];
if exist([filename '_feat.deffects'], 'file')
    Fdeffects=load([filename '_feat.deffects']);
end
Fcauneff=[];
if exist([filename '_feat.cauneff'], 'file')
    Fcauneff=load([filename '_feat.cauneff']);
end
Fdcauneff=[];
if exist([filename '_feat.dcauneff'], 'file')
    Fdcauneff=load([filename '_feat.dcauneff']);
end
Fspouses=[];
if exist([filename '_feat.spouses'], 'file')
    Fspouses=load([filename '_feat.spouses']);
end
Frelatives=[];
if exist([filename '_feat.relatives'], 'file')
    Frelatives=load([filename '_feat.relatives']);
end
Fblanket=[];
if exist([filename '_feat.blanket'], 'file')
    Fblanket=load([filename '_feat.blanket']);
end

Fparents=[];
if exist([filename '_feat.parents'], 'file')
    Fparents=read_parents([filename '_feat.parents']);
end

if add_valid,
    set_name    = {'train', 'valid', 'test'};
else
    
    set_name    = {'train', 'test'};
end

% Read the Data
if ~exist(([filename '_' set_name{1} '.targets']), 'file')
    extent='.labels';
else
    extent='.targets';
end
for j = 1:length(set_name)
    ID=[];
    YM=[]; % Multiclass labels
    %YC=[]; % Continuous targets
    % read the data
    dataname=[filename '_' set_name{j}];
    if exist([dataname '.data'])== 2 | fcheck([dataname  '.mat']),
        if fcheck([dataname  '.mat'])
            load([dataname  '.mat']);            
        elseif read_as_sparse
            X = sparse_data_read([dataname '.data'], ...
                p.feat_num, ...
                p.([set_name{j} '_num']), ...
                p.data_type);
        else
        	X = matrix_data_read([dataname '.data'], ...
                p.feat_num, ...
                p.([set_name{j} '_num']), ...
                p.data_type);
        end
    elseif exist([dataname '.csv']) == 2,
        X = textread([dataname '.csv'], '%s', 'delimiter', ',');
        X = reshape(X,length(X)/p.([set_name{j} '_num']),p.([set_name{j} '_num']))';
    end
    % Save in Matlab format for faster reload
    %if ~fcheck([dataname  '.mat'])
    %    save([dataname  '_V6.mat'], 'X', '-V6'); 
    %end
    Y = read_targets([filename '_' set_name{j} extent]);
    if exist([filename '_' set_name{j} '.mlabels'], 'file')
        YM = textread([filename '_' set_name{j} '.mlabels'], '%s');
    end
    if exist([filename '_' set_name{j} '.id'], 'file')
        ID = textread([filename '_' set_name{j} '.id'], '%s');
        if length(ID)>length(Y)
            ID=reshape(ID,2,length(Y))';
        end
    end
    if isobject(data)
        Data.(set_name{j}) = data(X , Y); % Spider object
    else
        Data.(set_name{j}) = data(X , Y, YM, ID);
    end
    clear X; clear Y; clear YM; clear ID;
end
Data.Fid=Fid';
Data.Fsource=Fsource';
Data.Fcauses=Fcauses';
Data.Fdcauses=Fdcauses';
Data.Feffects=Feffects';
Data.Fdeffects=Fdeffects';
Data.Fcauneff=Fcauneff';
Data.Fdcauneff=Fdcauneff';
Data.Fparents=Fparents';
Data.Fspouses=Fspouses';
Data.Fblanket=Fblanket';
Data.Frelatives=Frelatives';

% Check the Data
if ~isfield(p, 'target_type'), p.target_type='binary'; end
if strcmp(p.target_type, 'binary') & isfield(p, 'train_pos_num')
    for j = 1:length(set_name) 
        check_labels(Data.(set_name{j}).Y, ...
                     p.([set_name{j} '_num']), ...
                     p.([set_name{j} '_pos_num']));
    end
end

if isfield(p, 'train_check_sum')
    for j = 1:length(set_name) 
        check_data(Data.(set_name{j}).X, p.([set_name{j} '_num']), ...
                   p.feat_num, ...
                   p.([set_name{j} '_check_sum']));
    end
end

return