% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-
%                       CAUSAL DISCOVERY CHALLENGE
%             Isabelle Guyon -- isabelle@clopinet.com -- December 2007
% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% DISCLAIMER: ALL INFORMATION, SOFTWARE, DOCUMENTATION, AND DATA ARE PROVIDED "AS-IS" 
% ISABELLE GUYON AND/OR OTHER ORGANIZERS DISCLAIM ANY EXPRESSED OR IMPLIED WARRANTIES, 
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
% FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY OF NON-INFRIGEMENT OF ANY THIRD PARTY'S 
% INTELLECTUAL PROPERTY RIGHTS. IN NO EVENT SHALL ISABELLE GUYON AND/OR OTHER ORGANIZERS 
% BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF SOFTWARE, DOCUMENTS, 
% MATERIALS, PUBLICATIONS, OR INFORMATION MADE AVAILABLE FOR THE CHALLENGE. 
%
% Code tested/corrected by Amir Saffari, Dec. 2007.
%
%% Initialization
clear all
%close all
%clc

% -o-|-o-|-o-|-o-|-o-|-o-|-o- BEGIN USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% 1) User-defined directories (no slash at the end of the names):
% --------------------------------------------------------------
my_root= '..'; % Change this to the directory where you put your code and data
data_dir    = [my_root '/Data']; % A directory containing subdirectories with DALTON, etc.
resu_dir    = [my_root '/Resu'];  
score_dir    = [my_root '/Score'];  
zip_dir     = [my_root '/Zipped'];  % Zipped files with results ready to go!
model_name  ='Sample';
% If you want, you can download the CLOP package from
% http://clopinet.com/CLOP, but this is optional
% Put the path to CLOP here, e.g.
%clop_dir=[my_path '/Clop/'];
clop_dir=[];
                                   
% 2) Choose your training data sets
% ---------------------------------
dataset     = {'lucas0', 'lucas1', 'lucas2', 'lucap0', 'lucap1', 'lucap2', 'reged0', 'reged1', 'reged2', 'sido0', 'sido1', 'sido2'}; 

%% 3) Advanced %%
% ---------------
KMAX=[];        % Number of classifiers you want to train
FNUM=[];        % Number of features if you train a single classifier (KMAX=1)
MBFIRST=0;      % Put forward all direct causes and non manipulated direct consequences and spouses
                % (can be used only if you ahve the truth values)
SAVEAS='slist'; % You can save as 'ulist' or 'slist' (if you use a ulist, you cannot turn in several preditions)
if ~isempty(clop_dir)
    MY_MODEL=chain({kridge, bias}); % A CLOP model (if you use CLOP)
else
    MY_MODEL=[];
end
% -o-|-o-|-o-|-o-|-o-|-o-|-o- END USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-
fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n');

% Set the path and defaults properly; create directories
% ------------------------------------------------------
makedir(resu_dir);
makedir(score_dir);
makedir(zip_dir);

use_clop=0;
if ~isempty(clop_dir)
    warning off; addpath(clop_dir); warning on;
    if exist('use_spider_clop.m') == 2, 
        use_clop=use_spider_clop(clop_dir);
    elseif exist('code_version.m') == 2, 
        fprintf('Sample code version : %s\n', code_version('sample_code'));
    else disp 'ERROR: Wrong code path. Check your directories and path variables.';
        if exist('README.txt') == 2, type README.txt; end
        if exist('Data/README.txt') == 2, type Data/README.txt; end
        if exist('Clop/README.txt') == 2, type Clop/README.txt; end
        return; 
    end
end

% LOOP OVER DATASETS 
% ===================
for j = 1:length(dataset)
    
	data_name   = dataset{j};
    
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-      %s     -o-|-o-|-o-|-o-|-o-|-o-\n', upper(data_name));
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n\n');
        
    resu_name   = [resu_dir '/' model_name ];
    makedir(resu_name);
    resu_name   = [resu_name '/' data_name];
        
    % Create a data structure and check the data
    %===========================================
    fprintf('-- %s loading data --\n', upper(data_name));
    input_dir   = [data_dir '/' upper(data_name)];
    input_name  = [input_dir '/' data_name];
    read_as_sparse=0;
    if strfind(data_name, 'sido'), read_as_sparse=1; end
    D   = create_data_struct(input_name, [], read_as_sparse);
    set_name    = {'train', 'test'};
        
    % Compute data statistics
    data_stats(D); % Comment out to save time

    fprintf('-- %s data loaded --\n', upper(data_name));

    % Perform feature selection
    %==========================
    fprintf('\n-- %s-%s feature selection --\n', upper(data_name), upper(model_name));
    % This is a place holder.
    % You need to sort the variables in order of "causal relevance" to the target. 
    % Causal relevance depends on the type of manipulation performed in the
    % test set (e.g. direct causes may be most relevant in some cases, but
    % sometimes a combination of causes and effects is best.)
    N=size(D.train.X,2);
    if ~isempty(FNUM), my_fnum=FNUM; else my_fnum=N; end
    if ~use_clop
        %Here, we just put a random order:   
        my_order=randperm(N);
    else
        % This is a "fancier" example using CLOP (actually not doing any
        % causal discovery, just ranking the variables according to their
        % correlation to the target).
    	my_model=chain({subsample({'p_max=4000', 'balance=1'}), s2n});
        if ~MBFIRST
            [d, m]=train(my_model, data(D.train.X, D.train.Y));
            my_order=get_fidx(m)';
        else
            % This puts first the direct causes 
            % (possible only if you have the truth values handy)
            fprintf('** Putting most direct predictive varaibles **\n');
            dcidx=find(D.Fblanket==1); 
            [d, m]=train(my_model, data(D.train.X(:,dcidx), D.train.Y));
            my_dc_order=get_fidx(m)';         
            ndcidx=find(D.Fblanket==-1);
            [d, m]=train(my_model, data(D.train.X(:,ndcidx), D.train.Y));
            my_ndc_order=get_fidx(m)';            
            my_order=[dcidx(my_dc_order), ndcidx(my_ndc_order)];
        end
    end
    % Show the top ranking variables  
    if isfield(D, 'Fid') 
        if ~isempty( D.Fid )
            fprintf('** Top ranking variables: **\n');
            for kk=1:min(20, N)
                fprintf('%d:\t%s\n', my_order(kk), D.Fid{my_order(kk)});
            end
        end
    end    
    % Compute performances (if you have the truth values!)
    ranks=[];
    if ~isempty(D.Fcauses)
        fprintf('** Feature ranking performances (the whole list): **\n');
        ranks(my_order)=[1:N]; % Highest ranks indicate irrelevant features
        print_fscores(ranks, D);
        if my_fnum~=N
            fprintf('** Feature ranking performances (only the top %d features treated as an slist): **\n', my_fnum);
            ranks(my_order(1:my_fnum))=1:my_fnum; 
            ranks(my_order(my_fnum+1:N))=my_fnum+1;
            print_fscores(ranks, D);
            fprintf('** Feature ranking performances (only the top %d features treated as a ulist): **\n', my_fnum);
            ranks(my_order(1:my_fnum))=-1; 
            ranks(my_order(my_fnum+1:N))=1;
            print_fscores(ranks, D);
        end
    end
    
    % Build predictive models
    %========================
    fprintf('\n-- %s-%s building  models --\n', upper(data_name), upper(model_name));
    my_model0=MY_MODEL;
    my_model={};
    % This is a place holder.   
    % You need to create models, which make predictions of the target
    % for nested subsets of 1, 2, ... N features, using my_order,
    % or train a single classifier on a given subset of features.  
    if ~isempty(KMAX), kmax=KMAX; else kmax=ceil(log2(N))+1; end
    if kmax==1, % a single classifier, train on all the features
        kmax=ceil(log2(my_fnum))+1;
        kmin=kmax;
    else
        kmin=1;
    end
    i=1;
    knum=kmax-kmin+1;
    for kk=kmin:kmax
        fn=min( 2^(kk-1), my_fnum);
        fprintf('** using %d variable(s) **\n', fn);
        if use_clop
            % Here an example using CLOP:
            [d, my_model{i}] = train(my_model0, data(D.train.X(:,my_order(1:fn)), D.train.Y));
            i=i+1;
            % replace by your own code...
        end
    end
    
    fprintf('-- %s-%s testing models\n', upper(data_name), upper(model_name));
    % This is a place holder, here make predictions using your models.
    ptr=size(D.train.X,1);
    pte=size(D.test.X,1);
    Train_predict=zeros(ptr, knum);
    Test_predict=zeros(pte, knum);
    fprintf('Num. var\tDAUC\tDERR\tTAUC\tTERR\n');
    i=1;
    for kk=kmin:kmax
        fn=min( 2^(kk-1), my_fnum);
        if use_clop
            % Here is an example using CLOP:
            d=test(my_model{i}, data(D.train.X(:,my_order(1:fn)), D.train.Y));
            Train_predict(:,i)=d.X;
            d=test(my_model{i}, data(D.test.X(:,my_order(1:fn)), D.test.Y));
            Test_predict(:,i)=d.X;
        else
            % We just copy the target values for the training data
        	Train_predict(:,i)=D.train.Y;
            % For test data,we randomly choose values from the training targets
            Y=D.train.Y;
            RandPred=Y(mod(randperm(pte),ptr)+1);
            Test_predict(:,i)=RandPred;
        end
        % Compute performances
        [a, e]=auc(Train_predict(:,i), D.train.Y);
        fprintf('%d\t%5.4f\t%5.4f\t', fn, a, e);
        if ~isempty(D.test.Y)
            [a, e]=auc(Test_predict(:,i), D.test.Y);
            fprintf('%5.4f\t%5.4f\n', a, e);
        end
        i=i+1;
    end
       
    % Save the results
    %=================
    fprintf('-- %s-%s saving the results\n', upper(data_name), upper(model_name));
    save_outputs([resu_name '_train.predict'], Train_predict);   
    save_outputs([resu_name '_test.predict'], Test_predict);
    save_outputs([resu_name '_feat.' SAVEAS], my_order(1:my_fnum)); % Use this to save a sorted list
    % Note: you should submit one slist OR one ulist, not both. If you have an
    % ordered list it is best to submit it as an slist, you will get a better score.
    fprintf('-- %s-%s results saved as %s*\n', upper(data_name), upper(model_name), resu_name);

    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n');

end % Loop over datasets

% Create a zip archive ready to go:
if ~usejava('jvm'), warning('Java is not loaded, failed to generate ZIP files !!!'); return; end
zip_name    = zipall(model_name, resu_dir, zip_dir);
if ~isempty(zip_name)
	fprintf('-- %s zip archive created, see %s --\n', upper(model_name), zip_name);
end

% Summarize all your method results in a table
simple_score(resu_dir, data_dir, score_dir);

