function simple_score(resudir, truthdir, scoredir)
%simple_score(resudir, truthdir, scoredir)
% Function to score the results of the benchmark, without sorting them
% resudir -- where the results are (.resu, .conf, .predict files)
% truthdir -- where the truth labels are
% scoredir -- where to store the scores

% Isabelle Guyon -- September 2003 -- isabelle@clopinet.com

% DISCLAIMER: ALL INFORMATION, SOFTWARE, DOCUMENTATION, AND DATA ARE PROVIDED "AS-IS" 
% THE ORGANIZERS DISCLAIMS ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
% THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. 
% IN NO EVENT SHALL ISABELLE GUYON AND/OR OTHER BENCHMARK ORGANIZERS BE LIABLE FOR ANY SPECIAL, 
% INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER ARISING OUT OF OR IN CONNECTION 
% WITH THE USE OR PERFORMANCE OF BENCHMARK SOFTWARE, DOCUMENTS, MATERIALS, PUBLICATIONS, OR 
% INFORMATION MADE AVAILABLE. 

fout=fopen([scoredir '/Simple-' date '.score'],'w') ;


% Identify the method, dataset, and result types:
[methodir, dataset, method] = identify(resudir, 0);

if fcheck([truthdir '/' upper(dataset{1}) '/' dataset{1} '_train.labels'])
    trainext='.labels';
else
    trainext='.targets';
end

validset=[];
validext=[];
if fcheck([truthdir '/' upper(dataset{1}) '/' dataset{1} '_valid' trainext]) 
    validset='valid';
    validext=trainext;
end

testset=[];
testext=[];
if fcheck([truthdir '/' upper(dataset{1}) '/' dataset{1} '_test' trainext]) 
    testset='test';
    testext=trainext;
end

featset=[];
featext=[];
if fcheck([truthdir '/' upper(dataset{1}) '/' dataset{1} '_feat.labels']) 
    featset='feat';
    featext='.labels';
end
if fcheck([truthdir '/' upper(dataset{1}) '/' dataset{1} '_feat.blanket'])
    featset='feat';
    featext='.blanket';
end

resuset={};
resuset=[resuset testset; validset; 'train' ; featset  ]; 
ext={};
ext=[ext testext;  validext; trainext; featext]; 

Nd=length(dataset);
Nm=length(methodir);

fprintf('Found %d datasets and %d methods\n', Nd, Nm);

% Loop over datasets and methods:
for i=1:Nd
    if ~isempty(featset)
        fprintf(fout, '\n** %s ** \nmethod\t| Sorted?\tMaxed? |\t| FnumAUC\tMBAUC\t(DnumAUC)\tDAUC\t(TnumAUC)\tTAUC |\t| FnumBAC\tMBBAC\tDnumBAC\tDBAC\tTnumBAC\tTBAC |\n', upper(dataset{i}));
    else
        % Supposed to be backward compatibel, but not tested
        fprintf(fout, '\n** %s ** \nmethod\tDnum\tTnum\t', upper(dataset{i}));
        for k=1:length(resuset)
            fprintf(fout, '%s_auc\t', resuset{k});
        end
        for k=1:length(resuset)
            fprintf(fout, '%s_bac\t', resuset{k});
        end
    end

    fprintf(fout, '\n');
    for j=1:length(methodir)
        bacval=-1*ones(length(resuset),1);
        aucval=-1*ones(length(resuset),1);
        bacbar=-1*ones(length(resuset),1);
        aucbar=-1*ones(length(resuset),1);
        pos_num_auc=-1*ones(length(resuset),1);
        pos_num_bac=-1*ones(length(resuset),1);
        frac_feat=-1;
        fsorted=0; % unsorted features
        maxed=0; % max of .predict taken
        ai_auc=0; % value of the column index at max AUC
        ai_bac=0; % value of the column index at max BAC
        used_feat_num_auc=0;
        used_feat_num_bac=0;
        
        for k=1:length(resuset)
            % Get the truth values (note: the 'feat' also have truth values)
            Y=fload([truthdir '/' upper(dataset{i}) '/' dataset{i} '_' resuset{k} ext{k}]);
            % Get the results
            Y_resu=[];
            F_resu=[];
            Y_conf=[];
            Y_score=[];
            F_score=[];
            ranks=[];
            tranks=[];
            if ~strcmp(resuset{k}, 'feat')
                % Get the outputs (backward compatible)
                Y_resu=fload([resudir '/' methodir{j} '/' dataset{i} '_' resuset{k} '.resu']);
                if isempty(Y_resu)
                    Y_score=fload([resudir '/' methodir{j} '/' dataset{i} '_' resuset{k} '.predict']);
                    Y_resu=sign(Y_score);
                    Y_resu(find(Y_resu==0))=1;
                else
                    Y_conf=fload([resudir '/' methodir{j} '/' dataset{i} '_' resuset{k} '.conf']);
                    if ~isempty(Y_conf) Y_score=Y_conf.*Y_resu; end
                end
            else
                if (fcheck([resudir '/' methodir{j} '/' dataset{i} '_feat.slist']) & fcheck([resudir '/' methodir{j} '/' dataset{i} '_feat.ulist']))
                    error(['Can''t get both slist and ulist for ' resudir '/' methodir{j} '/' dataset{i} '_feat.Xlist']);
                end
                F_resu=fload([resudir '/' methodir{j} '/' dataset{i} '.feat']);
                if isempty(F_resu), 
                    F_resu=fload([resudir '/' methodir{j} '/' dataset{i} '_feat.slist']);
                    if ~isempty(F_resu)
                        fsorted=1;
                    end % sorted features
                end
                if isempty(F_resu), 
                    F_resu=fload([resudir '/' methodir{j} '/' dataset{i} '_feat.ulist']);
                end
                if isempty(F_resu),
                    F_resu=1:length(Y); 
                end % Assume all the features have been used
               % F_resu
            end
            % Compute the performance
            if ~isempty(Y_resu) & ~isempty(Y)
                [a, e]=bac(Y_resu, Y);
                if k==1, 
                    if size(a,2)>1
                        maxed=1;
                    end
                    bv=max(a);
                    ai_bac=min(find(a==bv));
                end %store the max found for the test set
                bacval(k)=a(ai_bac);
                bacbar(k)=e(ai_bac);
                pos_num_bac(k)=length(find(Y_resu(:,ai_bac)>=0));
                if ~isempty(Y_score)
                    [a, e]=auc(-Y_score, -Y);
                    if k==1, 
                        av=max(a);
                        ai_auc=min(find(a==av));
                    end %store the max found for the test set
                    aucval(k)=a(ai_auc);
                    aucbar(k)=e(ai_auc);
                    pos_num_auc(k)=length(find(Y_resu(:,ai_auc)>=0));
                end
            elseif ~isempty(F_resu) & ~isempty(Y)
                all_feat_num=length(Y);
                given_feat_num=length(F_resu);
                used_feat_num_auc=given_feat_num;
                used_feat_num_bac=given_feat_num;
                if maxed
                    if fsorted
                        used_feat_num_auc=min(2^(ai_auc-1), given_feat_num);
                        used_feat_num_bac=min(2^(ai_bac-1), given_feat_num);
                    else
                        error('Multiple predictions for an unsorted list of features');
                    end
                end
                if fsorted
                    ranks(F_resu(1:given_feat_num))=1:given_feat_num; 
                    ranks(setdiff(1:all_feat_num, F_resu(1:given_feat_num)))=given_feat_num+1;   
                else
                    ranks(F_resu(1:given_feat_num))=-1; 
                    ranks(setdiff(1:all_feat_num, F_resu(1:given_feat_num)))=1;
                end
                tranks(F_resu(1:used_feat_num_bac))=-1; 
                tranks(setdiff(1:all_feat_num, F_resu(1:used_feat_num_bac)))=1;
                [aucval(k),aucbar(k)] = auc(ranks, -Y);
                [bacval(k),bacbar(k)] = bac(tranks, -Y);
                pos_num_auc(k)=used_feat_num_auc;
                pos_num_bac(k)=used_feat_num_bac;
            end
        end % for k

        if pos_num_auc(1)~=-1
            if ~isempty(featset)
            	fprintf(fout, '%s\t%d\t%d', method{j}, fsorted, maxed);
            end
            for kk=length(aucval):-1:1              
                fprintf(fout, '\t%d\t%5.4f+-%5.4f',pos_num_auc(kk), aucval(kk), aucbar(kk));
            end
            for kk=length(bacval):-1:1
                fprintf(fout, '\t%d\t%5.4f+-%5.4f',pos_num_bac(kk), bacval(kk), bacbar(kk));
            end            
            fprintf(fout, '\n');
        end 
    end % for j
end % for i

fclose(fout);

        
    
